@@ -1,6 +1,6 @@
 from .face import Face, split_face, create_face_from_diagonals
 from .block import Block, rotate_block, reduce_blocks
-from .connectivity import get_outer_faces, find_matching_blocks, get_face_intersection, connectivity
+from .connectivity import get_outer_faces, find_matching_blocks, get_face_intersection, connectivity, connectivity_fast
 from .read import read_plot3D
 from .write import write_plot3D
 from .differencing import find_edges, find_face_edges
