@@ -1,10 +1,9 @@
 from typing import List, Dict, Tuple
-from itertools import combinations_with_replacement 
+from itertools import combinations_with_replacement
 import numpy as np
-from numpy.core.shape_base import block
 from .block import Block, rotate_block
 from .face import Face, create_face_from_diagonals, split_face
-from .connectivity import get_face_intersection
+from .connectivity import get_face_intersection, face_matches_to_dict
 from .write import write_plot3D
 from math import cos, radians, sin, sqrt, acos, radians
 from copy import deepcopy
@@ -55,7 +54,7 @@ def create_face(block:Block,imin:int,imax:int,jmin:int,jmax:int,kmin:int,kmax:in
 #     for i,f in enumerate(face_list):
 #         temp.append({'block_indx':block_indices[i], 'IMIN':f.IMIN, 'IMAX':f.IMAX, 'JMIN':f.JMIN, 'JMAX':f.JMAX,'KMIN':f.KMIN, 'KMAX':f.KMAX}) 
 #     return temp 
-def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str='k', rotation_axis:str='x',nblades:int=55):
+def periodicity(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Face], periodic_direction:str='k', rotation_axis:str='x',nblades:int=55):
     """This function is used to check for periodicity of the other faces rotated about an axis 
         The way it works is to find faces of a constant i,j, or k value
 
@@ -94,13 +93,20 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
     
     # Here we make a list of all the outer faces
     outer_faces_all = list() 
+    matched_faces_all = list()
     periodic_faces = list()      # This is the output of the code 
-
+    periodic_faces_export = list() 
     for o in outer_faces:
         face = create_face(blocks[o['block_index']], o['IMIN'], o['IMAX'], o['JMIN'], o['JMAX'], o['KMIN'], o['KMAX'])
         face.set_block_index(o['block_index'])
         outer_faces_all.append(face)
-
+    for m in matched_faces:
+        face1 = create_face(blocks[m['block1']['block_index']], m['block1']['IMIN'], m['block1']['IMAX'], m['block1']['JMIN'], m['block1']['JMAX'], m['block1']['KMIN'], m['block1']['KMAX'])
+        face2 = create_face(blocks[m['block2']['block_index']], m['block2']['IMIN'], m['block2']['IMAX'], m['block2']['JMIN'], m['block2']['JMAX'], m['block2']['KMIN'], m['block2']['KMAX'])
+        face1.set_block_index(m['block1']['block_index'])
+        face2.set_block_index(m['block2']['block_index'])
+        matched_faces_all.append(face1)
+        matched_faces_all.append(face2)
     # Debug
     # debug_per1 = create_face(blocks[2], 0, 40, 0, 100,0,0) 
     # debug_per1.set_block_index(2)
@@ -130,28 +136,32 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
                     block1_rotated = rotate_block(blocks[face1.blockIndex],rotation_matrix1)
                     block2 = blocks[face2.blockIndex]
                     #   Check periodicity
-                    periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                    df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
 
                     if len(periodic_faces_temp) == 0:
                         block1_rotated = rotate_block(blocks[face1.blockIndex],rotation_matrix2)
                         block2 = blocks[face2.blockIndex]
-                        periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                        df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
                         if len(periodic_faces_temp)>0:  # Save the data
                             outer_faces_to_remove.append(face1)
                             outer_faces_to_remove.append(face2)
                             outer_faces_to_remove.append(periodic_faces_temp[0])    # Make sure periodic faces are also removed from outer faces during the loop
                             outer_faces_to_remove.append(periodic_faces_temp[1])
                             periodic_faces.append(periodic_faces_temp)
+                            periodic_faces_export.append(face_matches_to_dict(face1,face2,block1_rotated,block2))
                             split_faces.extend(split_faces_temp)
                             periodic_found = True
+                            break
                     else:   # Save the data
                         outer_faces_to_remove.append(face1)
                         outer_faces_to_remove.append(face2)
                         outer_faces_to_remove.append(periodic_faces_temp[0])
                         outer_faces_to_remove.append(periodic_faces_temp[1])
                         periodic_faces.append(periodic_faces_temp)
+                        periodic_faces_export.append(face_matches_to_dict(face1,face2,block1_rotated,block2))
                         split_faces.extend(split_faces_temp)
                         periodic_found = True
+                        break
 
             elif periodic_direction.lower() == "j":
                 
@@ -161,28 +171,32 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
                     block1_rotated = rotate_block(blocks[face1.blockIndex],rotation_matrix1)
                     block2 = blocks[face2.blockIndex]
                     #   Check periodicity
-                    periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                    df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
 
                     if len(periodic_faces_temp) == 0:
                         block1_rotated = rotate_block(blocks[face1.blockIndex],rotation_matrix2)
                         block2 = blocks[face2.blockIndex]
-                        periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                        df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
                         if len(periodic_faces_temp)>0:  # Save the data
                             outer_faces_to_remove.append(face1)
                             outer_faces_to_remove.append(face2)
                             outer_faces_to_remove.append(periodic_faces_temp[0])
                             outer_faces_to_remove.append(periodic_faces_temp[1])
                             periodic_faces.append(periodic_faces_temp)
+                            periodic_faces_export.append(face_matches_to_dict(face1,face2,block1_rotated,block2))
                             split_faces.extend(split_faces_temp)
                             periodic_found = True
+                            break
                     else:   # Save the data
                         outer_faces_to_remove.append(face1)
                         outer_faces_to_remove.append(face2)
                         outer_faces_to_remove.append(periodic_faces_temp[0])
                         outer_faces_to_remove.append(periodic_faces_temp[1])
                         periodic_faces.append(periodic_faces_temp)
+                        periodic_faces_export.append(face_matches_to_dict(face1,face2,block1_rotated,block2))
                         split_faces.extend(split_faces_temp)
                         periodic_found = True
+                        break
 
             elif periodic_direction.lower() == 'k':       # constant k between surfaces 
                  if (face1.KMIN == face1.KMAX) and (face2.KMIN == face2.KMAX): 
@@ -191,28 +205,32 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
                     block1_rotated = rotate_block(blocks[face1.blockIndex],rotation_matrix1)
                     block2 = blocks[face2.blockIndex]
                     #   Check periodicity
-                    periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                    df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
 
                     if len(periodic_faces_temp) == 0:
                         block1_rotated = rotate_block(blocks[face1.blockIndex],rotation_matrix2)
                         block2 = blocks[face2.blockIndex]
-                        periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                        df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
                         if len(periodic_faces_temp)>0:  # Save the data
                             outer_faces_to_remove.append(face1)
                             outer_faces_to_remove.append(face2)
                             outer_faces_to_remove.append(periodic_faces_temp[0])
                             outer_faces_to_remove.append(periodic_faces_temp[1])
                             periodic_faces.append(periodic_faces_temp)
+                            periodic_faces_export.append(face_matches_to_dict(periodic_faces_temp[0],periodic_faces_temp[1],block1_rotated,block2))
                             split_faces.extend(split_faces_temp)
                             periodic_found = True
+                            break
                     else:   # Save the data
                         outer_faces_to_remove.append(face1)
                         outer_faces_to_remove.append(face2)
                         outer_faces_to_remove.append(periodic_faces_temp[0])
                         outer_faces_to_remove.append(periodic_faces_temp[1])
                         periodic_faces.append(periodic_faces_temp)
+                        periodic_faces_export.append(face_matches_to_dict(periodic_faces_temp[0],periodic_faces_temp[1],block1_rotated,block2))
                         split_faces.extend(split_faces_temp)
                         periodic_found = True
+                        break
 
         if (periodic_found):
             outer_faces_to_remove = list(set(outer_faces_to_remove))
@@ -226,6 +244,9 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
         outer_faces_to_remove.append(p[0])
         outer_faces_to_remove.append(p[1])
 
+    for m in matched_faces_all:
+        outer_faces_to_remove.append(m)
+
     outer_faces_to_remove = list(set(outer_faces_to_remove))    # Use only unique values
     outer_faces_all = [p for p in outer_faces_all if p not in outer_faces_to_remove]    # remove from outer faces 
     # remove any duplicate periodic face pairs 
@@ -238,29 +259,16 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
             if periodic_faces[i][1] == periodic_faces[j][0]:
                 if periodic_faces[i][0] == periodic_faces[j][1]:
                     indx_to_remove.append(j)
+    
+
+    periodic_faces_export = [periodic_faces_export[i] for i in range(len(periodic_faces)) if i not in indx_to_remove]
     periodic_faces = [periodic_faces[i] for i in range(len(periodic_faces)) if i not in indx_to_remove]
     # Export periodic faces and outer faces
-    periodic_faces_export = list() 
     outer_faces_export = list() 
 
-    for f in periodic_faces:
-        periodic_faces_export.append({
-                                'block1':{
-                                            'block_index':f[0].blockIndex,
-                                            'IMIN':f[0].IMIN,'JMIN':f[0].JMIN,'KMIN':f[0].KMIN,
-                                            'IMAX':f[0].IMAX,'JMAX':f[0].JMAX,'KMAX':f[0].KMAX
-                                        },
-                                'block2':{
-                                            'block_index':f[1].blockIndex,
-                                            'IMIN':f[1].IMIN,'JMIN':f[1].JMIN,'KMIN':f[1].KMIN,
-                                            'IMAX':f[1].IMAX,'JMAX':f[1].JMAX,'KMAX':f[1].KMAX
-                                        },
-                                })
-
     for o in outer_faces_all:
         outer_faces_export.append(o.to_dict())
                         
-
     return periodic_faces_export, outer_faces_export, periodic_faces, outer_faces_all
                         
 
@@ -337,14 +345,15 @@ def __periodicity_check__(face1:Face, face2:Face,block1:Block,block2:Block):
     Returns:
         (tuple): containing
 
+            - **df** (pandas.Dataframe): List of point matches for periodic surfaces 
             - **periodic_surface** (List[Face]):  These are faces that are periodic 
             - **split_surfaces** (List[Face]): Some blocks may have periodic faces with other blocks. But the faces may need to be split so say you pair a small face with a larger face. The split surfaces should be treated as an outer face  
-            - **outer_faces_to_remove** (List[int]): Indicies containing [face1_indx, face2_indx]. These are not needed anymore and split_blocks should be added to the list of outer faces 
 
     """
+    
     periodic_faces = list()
     split_faces = list()
-
+    swapped = False
     if (face2.diagonal_length < face1.diagonal_length): # switch so that face 2 is always longer
         temp = deepcopy(face1)
         face1 = deepcopy(face2)
@@ -352,21 +361,25 @@ def __periodicity_check__(face1:Face, face2:Face,block1:Block,block2:Block):
 
         temp_block = deepcopy(block1)
         block1 = deepcopy(block2)
-        block2 = temp_block
-
+        block2 = temp_block        
+        swapped = True
+    
     df,split_face1,split_face2 = get_face_intersection(face1,face2,block1,block2)
 
     if len(df)>4:
         f1 = create_face(block1,imin=df['i1'].min(),jmin=df['j1'].min(),kmin=df['k1'].min(), imax=df['i1'].max(),jmax=df['j1'].max(),kmax=df['k1'].max())
         f1.set_block_index(face1.blockIndex)
-        periodic_faces.append(f1)
 
         f2 = create_face(block2,imin=df['i2'].min(),jmin=df['j2'].min(),kmin=df['k2'].min(), imax=df['i2'].max(),jmax=df['j2'].max(),kmax=df['k2'].max())
         f2.set_block_index(face2.blockIndex)
-        periodic_faces.append(f2)
 
         split_faces.extend(split_face1)
         split_faces.extend(split_face2)
+        if swapped:
+            periodic_faces.append(f2)
+            periodic_faces.append(f1)
+        else:
+            periodic_faces.append(f1)
+            periodic_faces.append(f2)
 
-
-    return periodic_faces,split_faces
\ No newline at end of file
+    return df,periodic_faces,split_faces
\ No newline at end of file
